@@ -81,9 +81,9 @@ reclass = function(x, to, must_close) {
 get_lengths = function(x) {
 	switch(class(x)[1],
 		sfc_POINT = rep(1, length(x)),
-		sfc_MULTIPOINT = sapply(x, nrow),
-		sfc_LINESTRING = sapply(x, nrow),
-		sapply(x, length) # list
+		sfc_MULTIPOINT = vapply(x, nrow, 0L),
+		sfc_LINESTRING = vapply(x, nrow, 0L),
+		lengths(x) # list
 	)
 }
 
@@ -96,12 +96,12 @@ get_lengths = function(x) {
 #' Features that can't be cast to a single  MULTI* geometry are return as a 
 #' GEOMETRYCOLLECTION
 st_cast_sfc_default = function(x) {
-  if (!identical(unique(sapply(x, function(w) class(w)[3L])), "sfg"))
+  if (!identical(unique(vapply(x, function(w) class(w)[3L], "")), "sfg"))
     stop("list item(s) not of class sfg") # sanity check
   
   a <- attributes(x)
   ids = NULL
-  cls = unique(sapply(x, function(x) class(x)[2L]))
+  cls = unique(vapply(x, function(x) class(x)[2L], ""))
   if (length(cls) > 1) {
     if (all(cls %in% c("POINT", "MULTIPOINT"))) {
       x <- lapply(x, function(x) if (inherits(x, "POINT")) POINT2MULTIPOINT(x) else x)
@@ -148,7 +148,7 @@ st_cast.sfc = function(x, to, ..., ids = seq_along(x), group_or_split = TRUE) {
 		if (to == "POINT")
 			st_cast(st_cast(x, "MULTIPOINT"), "POINT")
 		else if (to %in% c("MULTIPOINT", "LINESTRING")) {
-			ret = lapply(x, function(y) structure(unlist(y), class = c(class(y)[1], to, "sfg")))
+			ret = lapply(x, function(y) structure(as.matrix(y), class = c(class(y)[1], to, "sfg")))
 			attributes(ret) = attributes(x)
 			reclass(ret, to, FALSE)
 		} else
