@@ -6,7 +6,8 @@
 Rcpp::NumericVector CPL_get_bbox(Rcpp::List sf, int depth = 0) {
 	Rcpp::NumericVector bb(4);
 	switch(depth) {
-		case 0: 
+
+		case 0: // points:
 		for (int i = 0; i < sf.size(); i++) {
 			Rcpp::NumericVector pt = sf[i];
 			if (i == 0) {
@@ -20,11 +21,15 @@ Rcpp::NumericVector CPL_get_bbox(Rcpp::List sf, int depth = 0) {
 			}
 		}
 		break;
-		case 1: // matrices
+
+		case 1: // list of matrices:
 		bb(0) = bb(1) = bb(2) = bb(3) = NA_REAL;
 		for (int i = 0; i < sf.size(); i++) {
 			Rcpp::NumericMatrix m = sf[i];
 			if (i == 0) { // initialize:
+				if (m.nrow() == 0)
+					return bb;
+					// Rcpp::stop("CPL_get_bbox: invalid geometry");
 				bb(0) = bb(2) = m(0,0);
 				bb(1) = bb(3) = m(0,1);
 			} 
@@ -36,19 +41,23 @@ Rcpp::NumericVector CPL_get_bbox(Rcpp::List sf, int depth = 0) {
 			}
 		}
 		break;
-		default:
+
+		default: // recursive list
+		bb(0) = bb(1) = bb(2) = bb(3) = NA_REAL;
 		for (int i = 0; i < sf.size(); i++) {
 			Rcpp::NumericVector bbi = CPL_get_bbox(sf[i], depth - 1); // recurse
-			if (i == 0) {
-				bb(0) = bbi(0);
-				bb(1) = bbi(1);
-				bb(2) = bbi(2);
-				bb(3) = bbi(3);
-			} else {
-				bb(0) = std::min(bbi(0),bb(0));
-				bb(1) = std::min(bbi(1),bb(1));
-				bb(2) = std::max(bbi(2),bb(2));
-				bb(3) = std::max(bbi(3),bb(3));
+			if (! Rcpp::NumericVector::is_na(bbi[0])) {
+				if (i == 0) {
+					bb(0) = bbi(0);
+					bb(1) = bbi(1);
+					bb(2) = bbi(2);
+					bb(3) = bbi(3);
+				} else {
+					bb(0) = std::min(bbi(0),bb(0));
+					bb(1) = std::min(bbi(1),bb(1));
+					bb(2) = std::max(bbi(2),bb(2));
+					bb(3) = std::max(bbi(3),bb(3));
+				}
 			}
 		}
 		break;
