@@ -19,8 +19,6 @@
 // Returns errors to R
 // Note only case 4 actually returns immediately
 // Lower error codes are recoverable
-// Compile with -DCONTINUE_ON_ERROR to ignore fatal errors
-// This may be needed when running R under a different main loop
 //
 static void __err_handler(CPLErr eErrClass, int err_no, const char *msg)
 {
@@ -37,9 +35,7 @@ static void __err_handler(CPLErr eErrClass, int err_no, const char *msg)
             break;
         case 4:
             Rf_warning("GDAL Error %d: %s\n", err_no, msg); // #nocov
-            #ifndef CONTINUE_ON_ERROR
             Rcpp::stop("Unrecoverable GDAL error\n"); // #nocov
-            #endif
             break;        
         default:
             Rf_warning("Received invalid error class %d (errno %d: %s)\n", eErrClass, err_no, msg); // #nocov
@@ -99,25 +95,44 @@ Rcpp::List CPL_crs_parameters(std::string p4s) {
 	srs->exportToWkt(&cp);
 	out(5) = Rcpp::CharacterVector::create(cp);
 	CPLFree(cp);
+	delete srs;
 	return out;
 }
 
 std::vector<OGRGeometry *> ogr_from_sfc(Rcpp::List sfc, OGRSpatialReference **sref) {
 	double precision = sfc.attr("precision");
-	Rcpp::List wkblst = CPL_write_wkb(sfc, false, native_endian(), get_dim(sfc), precision);
+	Rcpp::List wkblst = CPL_write_wkb(sfc, false, native_endian(), get_dim_sfc(sfc, NULL), precision);
 	std::vector<OGRGeometry *> g(sfc.length());
 	OGRGeometryFactory f;
 	OGRSpatialReference *local_srs = NULL;
 	Rcpp::List crs = sfc.attr("crs");
+	Rcpp::IntegerVector epsg(1);
+	epsg[0] = crs["epsg"];
 	Rcpp::String p4s = crs["proj4string"];
-	if (p4s != NA_STRING) {
+	if (epsg[0] != NA_INTEGER) {
+		local_srs = new OGRSpatialReference;
+		OGRErr err = local_srs->importFromEPSG(epsg[0]);
+		if (err != 0) {
+			local_srs->Release(); // #nocov
+			handle_error(err);    // #nocov
+		}
+	} else if (p4s != NA_STRING) {
 		Rcpp::CharacterVector cv = crs["proj4string"];
 		local_srs = new OGRSpatialReference;
-		handle_error(local_srs->importFromProj4(cv[0]));
+		OGRErr err = local_srs->importFromProj4(cv[0]);
+		if (err != 0) {
+			local_srs->Release(); // #nocov
+			handle_error(err);    // #nocov
+		}
 	}
 	for (int i = 0; i < wkblst.length(); i++) {
 		Rcpp::RawVector r = wkblst[i];
-		handle_error(f.createFromWkb(&(r[0]), local_srs, &(g[i]), r.length(), wkbVariantIso));
+		OGRErr err = f.createFromWkb(&(r[0]), local_srs, &(g[i]), r.length(), wkbVariantIso);
+		if (err != 0) {
+			if (local_srs != NULL)      // #nocov
+				local_srs->Release();   // #nocov
+			handle_error(err);          // #nocov
+		}
 	}
 	if (sref != NULL)
 		*sref = local_srs; // return and release later, or
@@ -191,8 +206,9 @@ Rcpp::List sfc_from_ogr(std::vector<OGRGeometry *> g, bool destroy = false) {
 		Rcpp::RawVector raw(g[i]->WkbSize());
 		handle_error(g[i]->exportToWkb(wkbNDR, &(raw[0]), wkbVariantIso));
 		lst[i] = raw;
+		OGRGeometryFactory f;
 		if (destroy)
-			delete g[i];
+			f.destroyGeometry(g[i]);
 	}
 	Rcpp::List ret = CPL_read_wkb(lst, false, native_endian());
 	ret.attr("crs") = crs;
@@ -240,12 +256,28 @@ Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::CharacterVector proj4, Rcpp::Inte
 
 	// transform geometries:
 	std::vector<OGRGeometry *> g = ogr_from_sfc(sfc, NULL);
+	if (g.size() == 0) {
+		dest->Release(); // #nocov
+		throw std::range_error("CPL_transform: zero length geometry list"); // #nocov
+	}
 	OGRCoordinateTransformation *ct = 
 		OGRCreateCoordinateTransformation(g[0]->getSpatialReference(), dest);
-	if (ct == NULL)
-		throw std::range_error("OGRCreateCoordinateTransformation() returned NULL: PROJ.4 available?");
-	for (size_t i = 0; i < g.size(); i++)
-		handle_error(g[i]->transform(ct));
+	if (ct == NULL) {
+		dest->Release(); // #nocov
+		throw std::range_error("OGRCreateCoordinateTransformation() returned NULL: PROJ.4 available?"); // #nocov
+	}
+	for (size_t i = 0; i < g.size(); i++) {
+		CPLPushErrorHandler(CPLQuietErrorHandler);
+		OGRErr err = g[i]->transform(ct);
+		CPLPopErrorHandler();
+		if (err == 1 || err == 6) {
+			OGRwkbGeometryType geomType = g[i]->getGeometryType();
+			OGRGeometryFactory f;
+			f.destroyGeometry(g[i]);
+			g[i] = f.createGeometry(geomType);
+		} else
+			handle_error(err);
+	}
 
 	Rcpp::List ret = sfc_from_ogr(g, true); // destroys g;
 	if (epsg[0] != NA_INTEGER) {
